@@ -1,9 +1,115 @@
-# tramnet main function
+#' Regularized transformation models
+#' @rdname tramnet
+#'
+#' @param model Either a \code{"formula"} specifying the regression or an object
+#'     of class \code{"tram"}.
+#' @param ... Additional arguments passed to \code{\link[CVXR]{solve}}.
+#'
+#' @return An object of class \code{"tramnet"} with \code{coef}, \code{logLik},
+#'     \code{summary}, \code{simulate}, \code{residuals} and \code{plot} methods
+#'
+#' @details Partially penalized and constrained transformation models,
+#'    including Cox models and continuous outcome logistic regression.
+#'    The methodology is described in the \code{tramnet} vignette
+#'    accompanying this package.
+#'
+#' @references Lucas Kook and Torsten Hothorn, The R Journal (2021) 13:1, pages
+#'    581-594. \doi{10.32614/RJ-2021-054}
+#'
+#' @examples
+#'
+#' if (require("penalized") & require("survival")) {
+#'   ## --- Comparison with penalized
+#'   data("nki70", package = "penalized")
+#'   nki70$resp <- with(nki70, Surv(time, event))
+#'   x <- scale(model.matrix( ~ 0 + DIAPH3 + NUSAP1 + TSPYL5 + C20orf46,
+#'                           data = nki70))
+#'   fit <- penalized(response = resp, penalized = x, lambda1 = 1, lambda2 = 0,
+#'                    standardize = FALSE, data = nki70)
+#'   y <- Coxph(resp ~ 1, data = nki70, order = 10, log_first = TRUE)
+#'   fit2 <- tramnet(y, x, lambda = 1, alpha = 1) ## L1 only
+#'   coef(fit)
+#'   coef(fit2)
+#' }
+#'
+#' @export
+tramnet <- function(model, ...) {
+  UseMethod("tramnet")
+}
 
-tramnet <-
-  function(model, x, lambda, alpha, constraints = NULL, ...) {
+#' @rdname tramnet
+#'
+#' @param data Object of class \code{"data.frame"} containing the variables
+#'    referred to in the formula \code{model}.
+#' @param tram_fun Character referring to an implementation in package
+#'    \code{'tram'}. See \code{\link[tramnet]{BoxCoxNET}} for the implemented
+#'    models.
+#' @param tram_args Additional arguments (besides \code{model} and \code{data})
+#'    passed to \code{tram_fun}.
+#'
+#' @exportS3Method tramnet formula
+#'
+tramnet.formula <- function(
+    model, data, lambda, alpha, tram_fun, tram_args = NULL, constraints = NULL,
+    groups = NULL, ...
+) {
+
+  call <- match.call()
+
+  ### Fit unpenalized unconditional model
+  fm0 <- update(model, . ~ 1)
+  m0 <- do.call(tram_fun, args = c(list(formula = fm0, data = data), tram_args))
+
+  ### Model matrix
+  x <- scale(.rm_int(model.matrix(model, data)))
+  preproc <- function(newdata) {
+    scale(.rm_int(model.matrix(model, newdata)),
+          center = attr(x, "scaled:center"),
+          scale = attr(x, "scaled:scale"))
+  }
+
+  ret <- tramnet.tram(model = m0, x = x, lambda = lambda, alpha = alpha,
+                      constraints = constraints, groups = groups, ...)
+  ret$process_newdata <- preproc
+  ret
+}
+
+.rm_int <- function(x) {
+  if (all(x[, 1] == 1))
+    return(x[, -1L, drop = FALSE])
+  return(x)
+}
+
+#' @rdname tramnet
+#' @param x A numeric matrix, where each row corresponds to the same row in the
+#'    \code{data} argument used to fit \code{model}.
+#' @param lambda A positive penalty parameter for the whole penalty function.
+#' @param alpha A mixing parameter (between zero and one) defining the fraction
+#'    between lasso and ridge penalties, where \code{alpha = 1} corresponds to
+#'    a pure lasso and \code{alpha = 0} to a pure ridge penalty.
+#' @param constraints An optional list containing a matrix of linear inequality
+#'    contraints on the regression coefficients and a vector specifying the rhs
+#'    of the inequality.
+#' @param groups For group lasso penalties, groups can be supplied as a vector
+#'    of consecutive integers of the same length as columns in \code{x}.
+#'
+#' @exportS3Method tramnet tram
+#'
+#' @import CVXR
+#' @import tram
+#' @importFrom basefun as.basis
+#' @importFrom grDevices rgb
+#' @importFrom graphics matplot par plot text
+#' @importFrom mlt `coef<-` as.mlt ctm mlt
+#' @importFrom sandwich estfun
+#' @importFrom stats as.formula coef getCall logLik model.matrix predict
+#'     simulate update variable.names weights update.default
+#'
+tramnet.tram <- function(model, x, lambda, alpha, constraints = NULL,
+                         groups = NULL, ...) {
     ### <FIXME> handle offset and maybe fixed parameters </FIXME>
-    .tramnet_checks(model = model, x = x, lambda = lambda, alpha = alpha)
+    .tramnet_checks(model = model, x = x, lambda = lambda, alpha = alpha,
+                    groups = groups)
     call <- match.call()
     trdat <- .get_tram_data(model)
     stopifnot(trdat$nobs == nrow(x))
@@ -12,7 +118,7 @@ tramnet <-
     theta <- Variable(nth)
     beta <- Variable(nb)
     prob <- .tramnet_objective(trdat, x, theta, beta,
-                               alpha, lambda, constraints)
+                               alpha, lambda, constraints, groups)
     res <- solve(prob, ...)
     ret <- list(call = call, model = model, x = x, result = res,
                 beta = res$getValue(beta), theta = res$getValue(theta),
@@ -22,7 +128,9 @@ tramnet <-
     return(ret)
   }
 
-.tramnet_checks <- function(model, x, lambda, alpha) {
+.tramnet_checks <- function(model, x, lambda, alpha, groups) {
+  if (!is.null(groups) && !all(diff(sort(unique(groups))) == 1))
+    stop("The provides `groups` should be supplied as consecutive integers.")
   if (!(inherits(model, "tram") | inherits(model, "mlt")))
     stop("The provided model should be of class 'tram'")
   if (!inherits(x, "matrix"))
@@ -40,7 +148,7 @@ tramnet <-
 }
 
 .tramnet_objective <-
-  function(trdat, x, theta, beta, alpha, lambda, constraints) {
+  function(trdat, x, theta, beta, alpha, lambda, constraints, groups) {
     xe <- x[trdat$exact$which, , drop = FALSE]
     xl <- x[trdat$censl$which, , drop = FALSE]
     xr <- x[trdat$censr$which, , drop = FALSE]
@@ -155,9 +263,18 @@ tramnet <-
         lll <- llr <- lli <- 0
       }
     )
-    obj <- -(lle + lll + llr + lli) +
-      (0.5 * (1 - alpha) * power(p_norm(lambda * beta, 2), 2) +
-                  alpha * p_norm(lambda * beta, 1))
+    lik <- -(lle + lll + llr + lli)
+    if (is.null(groups))
+      pen <- (0.5 * (1 - alpha) * power(p_norm(lambda * beta, 2), 2) +
+                alpha * p_norm(lambda * beta, 1))
+    else {
+      pens <- sapply(unique(groups), \(group) {
+        pmat <- diag(as.numeric(groups == group))
+        p_norm(pmat %*% beta, 2)
+      })
+      pen <- lambda * do.call("sum", pens)
+    }
+    obj <- lik + pen
     const <- list(trdat$const$ui %*% theta >= trdat$const$ci)
     if (!is.null(constraints)) {
       const[[2]] <- constraints[[1]] %*% beta >= constraints[[2]]
@@ -281,3 +398,4 @@ tramnet <-
     return(c("tramnet_Lm", "tramnet"))
   return("tramnet")
 }
+
